Module: FunWith::Files::CoreExtensions::File
- Defined in:
- lib/fun_with/files/core_extensions/file.rb
Instance Method Summary collapse
-
#fwf_blank? ⇒ Boolean
I’m not sure this is the most intuitive meaning, but it seems better than delegating to Object.
- #fwf_filepath(*args) ⇒ Object
Instance Method Details
#fwf_blank? ⇒ Boolean
I’m not sure this is the most intuitive meaning, but it seems better than delegating to Object.
11 12 13 |
# File 'lib/fun_with/files/core_extensions/file.rb', line 11 def fwf_blank? ! self.fwf_filepath.exist? end |
#fwf_filepath(*args) ⇒ Object
5 6 7 |
# File 'lib/fun_with/files/core_extensions/file.rb', line 5 def fwf_filepath( *args ) FunWith::Files::FilePath.new( self.path, *args ) end |